CLEAR
SET DEFAULT TO (LEFT(SYS(16), RAT("\", SYS(16))))

#INCLUDE "include.h"

LOCAL lbResult as Boolean;
	,lbError as Boolean
TRY
	*/////////////////////////////////////////////////
	*// Check external files
	LOCAL lcFileTemplateDocx as String
	lcFileTemplateDocx = LOWER(FULLPATH(PATH_DOCX)) + TEMPLATE_DOCX
	IF !FILE(lcFileTemplateDocx)
		ERROR (ERR_FILE_NOTFOUND + lcFileTemplateDocx)
	ENDIF
	LOCAL lcFileGenXsltFromTemplateXslt as String 
	lcFileGenXsltFromTemplateXslt = LOWER(FULLPATH(PATH_XSLT)) + XSLT_FILE
	IF !FILE(lcFileGenXsltFromTemplateXslt)
		ERROR (ERR_FILE_NOTFOUND + lcFileGenXsltFromTemplateXslt)
	ENDIF
	LOCAL lcFileXmlData as String 
	lcFileXmlData = LOWER(FULLPATH(PATH_XML)) + XML_DATA_FILE
	IF !FILE(lcFileGenXsltFromTemplateXslt)
		ERROR (ERR_FILE_NOTFOUND + lcFileXmlData)
	ENDIF
	LOCAL lcTempDir as String
	lcTempDir = LOWER(FULLPATH(PATH_TMP))
	IF !DIRECTORY(lcTempDir)
		ERROR (ERR_DIR_NOTFOUND + lcTempDir)
	ENDIF
	LOCAL lcUnpackDir as String
	lcUnpackDir = JUSTSTEM(TEMPLATE_DOCX)
	LOCAL lcTempContentDocxDir as String
	lcTempContentDocxDir = ADDBS(lcTempDir + lcUnpackDir)
	IF !DIRECTORY(lcTempContentDocxDir)
		ERROR (ERR_DIR_NOTFOUND + lcTempContentDocxDir + "." ;
			+ CRLF + ERR_USER_USE_PROG1 + "step1_unpack.prg" + ERR_USER_USE_PROG2)
	ENDIF

	*//////////////////////////////////////////////////////////////
	*// Get FileDocumentXml as FullFilename to file document.xml
	LOCAL lcFileDocumentXml as String
	lcFileDocumentXml = lcTempDir + lcUnpackDir + CHRTRANC(URI_DOCUMENT_XML, '/', '\')
	IF !FILE(lcFileDocumentXml)
		ERROR (ERR_FILE_NOTFOUND + lcFileDocumentXml)
	ENDIF

	*//////////////////////////////////////////////////////////////
	*// Remove lcFileTemplateDocxTmp in temp folder
	LOCAL lcFileTemplateDocxTmp as String 
	lcFileTemplateDocxTmp = lcTempDir + TEMPLATE_DOCX
	IF FILE(lcFileTemplateDocxTmp)
		ERASE (lcFileTemplateDocxTmp)
	ENDIF	

	*///////////////////////////////////////////////////////////////////////////////
	*// Prepare and execute the first xslt-transformation 
	*// with document.xml and by using xslt-file FileGenXsltFromTemplate
	LOCAL ARRAY laParams[2,2]
	laParams[1,1] = "prmRootElement"
	laParams[1,2] = "VFPData"
	laParams[2,1] = "prmEncoding"
	laParams[2,2] = ENCODING
	LOCAL lbIsTransform as Boolean;
		,lcXsltOutput as String  
	lbIsTransform = XsltTransform(lcFileDocumentXml, lcFileGenXsltFromTemplateXslt;
		,@lcXsltOutput, @laParams)
	IF !lbIsTransform
		ERROR (ERR_XSLT_TRANSFORM)
	ENDIF
#IF TMP_OUT
	LOCAL lcPathCurDir as String;
		,lcFileFirstResultXslt as String 
	lcPathCurDir = LOWER(ADDBS(FULLPATH(CURDIR())))
	lcFileFirstResultXslt = lcPathCurDir + XSLT_FILE_FIRST_RESULT
	IF FILE(lcFileFirstResultXslt)
		ERASE (lcFileFirstResultXslt)
	ENDIF
	STRTOFILE(lcXsltOutput, lcFileFirstResultXslt)
#ENDIF

	*///////////////////////////////////////////////////////////////////////////////
	*// Prepare and execute the second xslt-transformation 
	*// with input xml-data-file and by using first result of xslt-transformation
	LOCAL lcXmlOutput as String
	lbIsTransform = .F.
	lbIsTransform = XsltTransform(lcFileXmlData, lcXsltOutput;
		,lcFileDocumentXml)
	IF !lbIsTransform
		ERROR (ERR_XSLT_TRANSFORM)
	ENDIF
#IF TMP_OUT
	LOCAL lcFileSecondResultXml as String 
	lcFileSecondResultXml = lcPathCurDir + JUSTFNAME(lcFileDocumentXml)
	IF FILE(lcFileSecondResultXml)
		ERASE (lcFileSecondResultXml)
	ENDIF
	COPY FILE (lcFileDocumentXml) TO (lcFileSecondResultXml)
#ENDIF
CATCH TO loException
	lbError = .T.
	LOCAL lcMsg as String
	lcMsg = ERR_CODE + LTRIM(STR(loException.ErrorNo)) + CRLF ;
		+ ERR_LINE + LTRIM(STR(loException.LineNo)) + CRLF ;
		+ ERR_MESSAGE + IIF(EMPTY(loException.Message), 'Unknown error', loException.Message)
	? ERR_LABEL_OUTPUT
	? lcMsg
	SET MESSAGE TO
	RETURN TO MASTER
FINALLY
	IF !lbError
		? LOWER(PROGRAM())+ ": " + MSG_USER_SUCCESS_STEP
		IF PROGRAM(-1) = 1
			? MSG_USER_NEXT_STEP + "step3_pack.prg"
		ENDIF
	ELSE
		? ERR_TERMINATED		
	ENDIF	
ENDTRY
RETURN lbResult

*/////////////////////////////////////////////////
*// XsltTransform
FUNCTION XsltTransform
	LPARAMETERS tcXml, tcXsl, tcOut, taParams
	EXTERNAL ARRAY taParams
	LOCAL lcXmlFile as String;
		,lcXslFile as String;
		,lcHtmFile as String;
		,lbResult as Boolean;
		,lbIsXmlFile as Boolean;
		,lbIsXslFile as Boolean;
		,lbIsOutFile as Boolean

	*/////////////////////////////////////////////////
	*// Check parameters
	lbIsXmlFile = (ATC('<', tcXml) = 0)   
	lbIsXslFile = (ATC('<', tcXsl) = 0)
	lbIsOutFile = (VARTYPE(tcOut) = 'C' AND ATC('.', tcOut) > 0)  
	IF lbIsXmlFile
		lcXmlFile = LOWER(FULLPATH(tcXml))
		IF !FILE(lcXmlFile)
			ERROR 101, lcXmlFile
			RETURN lbResult
		ENDIF
	ENDIF
	IF lbIsXslFile
		lcXslFile = LOWER(FULLPATH(tcXsl))
		IF !FILE(lcXslFile)
			ERROR 101, lcXslFile
			RETURN lbResult
		ENDIF
	ENDIF	
	IF lbIsOutFile
		lcOutFile = LOWER(FULLPATH(tcOut))
	ENDIF

	*/////////////////////////////////////////////////
	*// Create MSXSL objects
	LOCAL lbParseError as Boolean;
		,lcPROGIDmsxml as String;
		,lcPROGIDmsxsl as String;
		,lcPROGIDmstemp as String;
		,loMsXml as Msxml2.DOMDocument;
		,loMsXsl as Msxml2.FreeThreadedDOMDocument;
		,loMsPrc as Msxml2.XSLTemplate;
		,lobjXSLTProc as Msxml2.IXSLProcessor
	lcPROGIDmsxml = "Msxml2.DOMDocument" + XML_VERS
	lcPROGIDmsxsl = "Msxml2.FreeThreadedDOMDocument" + XML_VERS
	lcPROGIDmstemp = "Msxml2.XSLTemplate" + XML_VERS
	loMsXml = CREATEOBJECT(lcPROGIDmsxml)
	loMsXsl = CREATEOBJECT(lcPROGIDmsxsl)
	loMsPrc = CREATEOBJECT(lcPROGIDmstemp)

	*/////////////////////////////////////////////////
	*// Load XML & XSL
	*
	*-- Load XML
	WITH loMsXml 
		.async = .F.
		IF lbIsXmlFile
			.load(lcXmlFile) && Load XML-file
		ELSE
			.loadXML(tcXml) && Load XML-string
		ENDIF	
		IF .parseError.errorCode != 0
			lbParseError = .T.
			ShowXmlErr(.parseError)
		ENDIF
	ENDWITH
	IF !lbParseError
		*
		*-- Load XSL
		WITH loMsXsl
			.async = .F.
			IF lbIsXslFile
				.load(lcXslFile) && Load XSL-file
			ELSE
				.loadXML(tcXsl) && Load XSL-string
			ENDIF	
			IF .parseError.errorCode != 0
				lbParseError = .T.
				ShowXmlErr(.parseError)
			ENDIF
		ENDWITH
	ENDIF
	IF !lbParseError

		*/////////////////////////////////////////////////
		*// Craete Processor for transformation
		loMsPrc.stylesheet = loMsXsl.documentElement
		lobjXSLTProc = loMsPrc.createProcessor()
		IF VARTYPE(lobjXSLTProc) = "O"
			lobjXSLTProc.input = loMsXml

			*/////////////////////////////////////////////////
			*// Check & set xslt-parameters
			IF TYPE('taParams[1]') = 'C' AND !EMPTY(taParams[1])
				LOCAL lnCntPrms as Integer
				FOR lnCntPrms = 1 TO ALEN(taParams, 1)
					IF !EMPTY(taParams[lnCntPrms,1])
						lobjXSLTProc.addParameter(taParams[lnCntPrms,1], taParams[lnCntPrms,2], "")
					ENDIF
				ENDFOR 
			ENDIF
			
			*/////////////////////////////////////////////////
			*// Do XSLT transformation
			IF lobjXSLTProc.transform()
				LOCAL lcOutPut as String, lnPos as Integer 
				lcOutPut = lobjXSLTProc.output
				
				*/////////////////////////////////////////////////
				*// Prepare result of transformation
				lnPos = ATC('<', lcOutPut)
				IF lnPos > 0
					lcOutPut = SUBSTRC(lcOutPut, lnPos)
				ENDIF
				lnPos = ATC('<?xml', lcOutPut)
				IF lnPos = 0
					lcOutPut = XMLPI +  CRLF + lcOutPut
				ELSE
					lnPos = ATC('?>', lcOutPut)
					IF lnPos > 0
						lcOutPut = XMLPI + SUBSTRC(lcOutPut, lnPos + 2)
					ENDIF
				ENDIF

				*///////////////////////////////////////////////////////
				*// Converts output result by use established encoding
#IF OUTPUT_ENCODING
				LOCAL lcCurEncoding
				lcCurEncoding = LOWER(ENCODING)
				DO CASE
				CASE lcCurEncoding = 'windows-1251'
					*-- No any converts
				CASE lcCurEncoding = 'utf-8'
					lcOutPut = STRCONV(lcOutPut, 9)
				*!* CASE ENCODING = ... && <- insert your encoding converts here...!
				ENDCASE	
#ENDIF				
				IF lbIsOutFile
					*-- ... Save result as file
					IF FILE(lcOutFile)
						ERASE (lcOutFile)
					ENDIF
					LOCAL lnBytes as Number
					lnBytes = 0
					lnBytes = STRTOFILE(lcOutPut, lcOutFile)
					IF lnBytes > 0 AND FILE(lcOutFile)
						lbResult = .T.
					ELSE
						ERROR 101, lcOutFile
					ENDIF
				ELSE
					*-- ... result as string
					tcOut = lcOutPut
					lbResult = .T.
				ENDIF
			ELSE
				ERROR "Error on XML + XSL => XML transformation"
			ENDIF
		ELSE
			ERROR "Can not createProcessor()"
		ENDIF
	ENDIF
	STORE NULL TO ;
		loMsXml, loMsXsl ,loMsPrc ,lobjXSLTProc
	RETURN lbResult
ENDFUNC

*/////////////////////////////////////////////////
*// ShowXmlErr
FUNCTION ShowXmlErr
	LPARAMETERS toXmlErr as Msxml2.IXMLDOMParseError
	IF VARTYPE(toXmlErr) # 'O'
		RETURN .F.
	ENDIF
	LOCAL lcMsg as String 
	WITH toXmlErr
		lcMsg = "Error loading '" + ALLTRIM(.url) + "'"
		lcMsg = lcMsg + CRLF + ERR_XSLT_CODE + TRANSFORM(.errorCode, '@0')
		IF !EMPTY(.reason)
			lcMsg = lcMsg + CRLF + ERR_XSLT_RESONE + ALLTRIM(.reason)
		ENDIF
		IF !EMPTY(.filepos)
			lcMsg = lcMsg + CRLF + ERR_XSLT_FILEPOS + TRANSFORM(.filepos)
		ENDIF
		IF !EMPTY(.linepos)
			lcMsg = lcMsg + CRLF + ERR_XSLT_LINEPOS + TRANSFORM(.linepos)
		ENDIF
		IF !EMPTY(.line)
			lcMsg = lcMsg + CRLF + ERR_XSLT_LINE + TRANSFORM(.line)
		ENDIF
		IF !EMPTY(.srcText)
			lcMsg = lcMsg + CRLF + ERR_XSLT_TEXT + ALLTRIM(.srcText)
		ENDIF
		MESSAGEBOX(lcMsg, 16, _SCREEN.Caption)
	ENDWITH
ENDFUNC
