<?xml version="1.0" encoding="utf-8"?>
<!-- file: XmlTableToHtmForADODB.xslt -->
<xsl:stylesheet version="1.0"
  xmlns="http://www.w3.org/1999/xhtml"
	xmlns:s='uuid:BDC6E3F0-6DA3-11d1-A2A3-00AA00C14882'
	xmlns:dt='uuid:C2F41010-65B3-11d1-A29F-00AA00C14882'
	xmlns:rs='urn:schemas-microsoft-com:rowset'
	xmlns:z='#RowsetSchema'    
	xmlns:msxsl="urn:schemas-microsoft-com:xslt"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	exclude-result-prefixes="s dt rs z msxsl xsl">

  <xsl:output method="html" version="4.0"
    encoding="utf-8" indent="no" 
    doctype-public="-//W3C//DTD HTML 4.01 Transitional//EN"
		doctype-system="http://www.w3.org/TR/html4/loose.dtd" />

	<xsl:variable name="varSchema">
		<xsl:copy-of select="/./*/*[local-name()='Schema']" />
	</xsl:variable>

	<!-- Parsing from root ... -->
	<xsl:template match="/">
    <html>
      <head>
        <xsl:text disable-output-escaping="yes"><![CDATA[<meta http-equiv="Content-Type" content="text/html; charset=utf-8">]]></xsl:text>
				<style type="text/css"><xsl:comment>
body
{font-family: Verdana, Arial, Helvetica, sans-serif;
font-size: 8.5pt;}
h4
{font-family: Verdana, Arial, Helvetica, sans-serif;
color: darkblue;
font-size: 9.0pt;}
th
{font-family: Verdana, Arial, Helvetica, sans-serif;
font-size: 8.5pt;
color: darkblue;
background-color: WhiteSmoke;}
td
{font-family: Verdana, Arial, Helvetica, sans-serif;
font-size: 8.5pt; text-align: left; vertical-align: top;}
.even
{font-family: Verdana, Arial, Helvetica, sans-serif;
font-size: 8.5pt;
background-color: Azure;}</xsl:comment>
				</style>
				<title>Result from ADODB data source</title>
      </head>
			<body>
				<center>
					<table border="1" cellspacing="0" cellpadding="2">
						<!-- For each row-element -->
						<xsl:for-each select="/./*/*[local-name()='data']/*[local-name()='row']">
							<xsl:if test="position()=1">
								<!-- In case first position: generate table row with columns headers -->
								<tr>
									<!-- For each column... -->
									<xsl:for-each select="@*">
										<th>
											<!-- Select name of current attribute -->
											<xsl:value-of select="local-name()" />
										</th>
									</xsl:for-each>
								</tr>
							</xsl:if>
							<!-- Define class to current row -->
							<xsl:variable name="varClass">
								<xsl:if test="position() mod 2">
									<xsl:text>even</xsl:text>
								</xsl:if>
							</xsl:variable>
							<!-- Generate table row with data... -->
							<tr>
								<!-- For each column... -->
								<xsl:for-each select="@*">
									<!-- Generate value for current attribute -->
									<xsl:call-template name="getColValue">
										<xsl:with-param name="prmClass" select="$varClass" />
										<xsl:with-param name="prmValue" select="." />
									</xsl:call-template>
								</xsl:for-each>
							</tr>
						</xsl:for-each>
					</table>
				</center>
      </body>
    </html>
  </xsl:template>

  <!-- Generate column value (<td>...</td>) -->
  <xsl:template name="getColValue">
    <xsl:param name="prmClass" />
    <xsl:param name="prmValue" />
    <td>
      <!--
			<xsl:attribute name="valign">
        <xsl:text>top</xsl:text>
      </xsl:attribute>
			 -->
      <xsl:if test="string-length($prmClass)>0">
        <!-- Add class attribute if need... -->
        <xsl:attribute name="class">
          <xsl:value-of select="$prmClass" />
        </xsl:attribute>
      </xsl:if>
      <!-- Get name of current attribute -->
      <xsl:variable name="varName">
        <xsl:value-of select="local-name()" />
      </xsl:variable>
      <!-- Try get type of current attribute value -->
			<xsl:variable name="varType">
				<!-- ... try get type from schema -->
				<xsl:choose>
					<xsl:when test="msxsl:node-set($varSchema)/.//*[local-name()='AttributeType' and @name=$varName]/*[local-name()='datatype']/@dt:type">
						<xsl:value-of select="msxsl:node-set($varSchema)/.//*[local-name()='AttributeType' and @name=$varName]/*[local-name()='datatype']/@dt:type" />
					</xsl:when>
					<!-- ... insert your code here for any other types... -->
					<xsl:otherwise>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:if test="$varType='decimal' or $varType='int' or $varType='dateTime' or $varType='date'">
				<!-- Add align attribute if need... -->
				<xsl:attribute name="style">
					<xsl:text>text-align: right;</xsl:text>
				</xsl:attribute>
			</xsl:if>
			<!-- Convert value according to type of current attribute value -->
      <xsl:choose>
        <xsl:when test="$varType='dateTime' or $varType='date'">
          <!-- ... convert value to russian date format -->
          <xsl:value-of select="msxsl:format-date($prmValue, 'dd.MM.yyyy')" />
        </xsl:when>
        <!-- ... insert your code here for any other types... -->
        <xsl:otherwise>
          <!-- ... otherwise: simple copy of current value -->
          <xsl:value-of disable-output-escaping="yes" select="$prmValue" />
        </xsl:otherwise> 
      </xsl:choose>
    </td>
  </xsl:template>

</xsl:stylesheet>