<?xml version="1.0" encoding="utf-8"?>
<!-- file: XmlTableToHtm.xslt -->
<xsl:stylesheet version="1.0"
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema"
		xmlns:xs="http://www.w3.org/2001/XMLSchema"
		xmlns:msxsl="urn:schemas-microsoft-com:xslt"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    exclude-result-prefixes="xsi xsd xs msxsl xsl">

  <xsl:output method="html" version="4.0"
    encoding="utf-8" indent="no" 
		doctype-public="-//W3C//DTD HTML 4.01 Transitional//EN"
		doctype-system="http://www.w3.org/TR/html4/loose.dtd" />

  <xsl:variable name="prmTableLoc">
    <xsl:text>таблица</xsl:text> 
  </xsl:variable>

  <!-- Test: is the first data element as element of xsd-schema -->
  <xsl:variable name="varIsSchema">
    <xsl:choose>
      <xsl:when test="local-name(/./*/*[1])='schema'">
        <xsl:text>true</xsl:text> 
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>false</xsl:text> 
      </xsl:otherwise> 
    </xsl:choose> 
  </xsl:variable>

  <xsl:variable name="varFirstDataElement">
    <xsl:choose>
      <xsl:when test="$varIsSchema='true'">
        <xsl:text>2</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>1</xsl:text>
      </xsl:otherwise> 
    </xsl:choose> 
  </xsl:variable>

  <!-- Get name of first data element as table name -->
  <xsl:variable name="varTable">
    <xsl:value-of select="local-name(/./*/*[number($varFirstDataElement)])" />
  </xsl:variable>

  <!-- Get xsd-schema if exist -->
  <xsl:variable name="varSchema">
    <xsl:if test="$varIsSchema='true'">
      <xsl:copy-of select="/./*/*[1]" />
    </xsl:if>
  </xsl:variable>

	<xsl:variable name="varTblTitle">
		<xsl:value-of select="$varTable" />
		<xsl:text> - </xsl:text>
		<xsl:value-of select="$prmTableLoc" />
	</xsl:variable>

	<!-- Parsing from root ... -->
  <xsl:template match="/">
    <html>
      <head>
        <xsl:text disable-output-escaping="yes"><![CDATA[<meta http-equiv="Content-Type" content="text/html; charset=utf-8">]]></xsl:text>
        <title>
					<xsl:value-of select="$varTblTitle"/>
				</title>
				<style type="text/css">
					<xsl:comment>
						body
						{font-family: Verdana, Arial, Helvetica, sans-serif;
						font-size: 8.5pt;}
						h4
						{font-family: Verdana, Arial, Helvetica, sans-serif;
						color: darkblue;
						font-size: 9.0pt;}
						th
						{font-family: Verdana, Arial, Helvetica, sans-serif;
						font-size: 8.5pt;
						color: darkblue;
						background-color: WhiteSmoke;}
						td
						{font-family: Verdana, Arial, Helvetica, sans-serif;
						font-size: 8.5pt; text-align: left; vertical-align: top;}
						.even
						{font-family: Verdana, Arial, Helvetica, sans-serif;
						font-size: 8.5pt;
						background-color: Azure;}
					</xsl:comment>
				</style>
      </head>
			<body>
				<h4 style="text-align: center;">
					<xsl:value-of select="$varTblTitle"/>
				</h4>
				<center>
					<table title ="{$varTblTitle}" border="1" cellspacing="0" cellpadding="2">
						<!-- For each elements with name=$varTable -->
						<xsl:for-each select="/./*/*[local-name()=$varTable]">
							<xsl:if test="position()=1">
								<!-- In case first position: generate table row with columns headers -->
								<tr>
									<!-- For each column... -->
									<xsl:for-each select="*">
										<th>
											<!-- Select name of element -->
											<xsl:value-of select="local-name()" />
										</th>
									</xsl:for-each>
								</tr>
							</xsl:if>
							<!-- Define class to current row -->
							<xsl:variable name="varClass">
								<xsl:if test="position() mod 2">
									<xsl:text>even</xsl:text>
								</xsl:if>
							</xsl:variable>
							<!-- Generate table row with data... -->
							<tr>
								<!-- For each column... -->
								<xsl:for-each select="*">
									<!-- Generate column value -->
									<xsl:call-template name="getColValue">
										<xsl:with-param name="prmClass" select="$varClass" />
										<xsl:with-param name="prmValue" select="." />
									</xsl:call-template>
								</xsl:for-each>
							</tr>
						</xsl:for-each>
					</table>
				</center>
      </body>
    </html>
  </xsl:template>

  <!-- Generate column value (<td>...</td>) -->
  <xsl:template name="getColValue">
    <xsl:param name="prmClass" />
    <xsl:param name="prmValue" />
    <td>
			<!--
			<xsl:attribute name="valign">
        <xsl:text>top</xsl:text>
      </xsl:attribute>
			-->
      <xsl:if test="string-length($prmClass)>0">
        <!-- Add class attribute if need... -->
        <xsl:attribute name="class">
          <xsl:value-of select="$prmClass" />
        </xsl:attribute>
      </xsl:if>
      <!-- Get name of current element -->
      <xsl:variable name="varName">
        <xsl:value-of select="local-name()" />
      </xsl:variable>
      <!-- Try get type of current element -->
      <xsl:variable name="varType">
        <xsl:choose>
          <xsl:when test="$varIsSchema='true'">
            <!-- ... try get type from xsd-schema -->
            <xsl:choose>
              <xsl:when test="msxsl:node-set($varSchema)/.//*[local-name()='element' and @name=$varName]/@type">
                <xsl:value-of select="msxsl:node-set($varSchema)/.//*[local-name()='element' and @name=$varName]/@type" />
              </xsl:when>
              <xsl:when test="msxsl:node-set($varSchema)/.//*[local-name()='element' and @name=$varName]/*[local-name()='simpleType']/*[local-name()='restriction']/@base">
                <xsl:value-of select="msxsl:node-set($varSchema)/.//*[local-name()='element' and @name=$varName]/*[local-name()='simpleType']/*[local-name()='restriction']/@base" />
              </xsl:when>
              <!-- ... insert your code here for any other types... -->
              <xsl:otherwise>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <!-- ... otherwise try get type according to name of current element -->
            <xsl:choose>
              <xsl:when test="$varName='birthdate'">
                <xsl:text>date</xsl:text>
              </xsl:when>
              <!-- ... insert your code here for any other types... -->
              <xsl:otherwise>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:otherwise> 
        </xsl:choose>
      </xsl:variable>
			<!-- Remove prefix from element type -->
			<xsl:variable name="varTypeValue">
				<xsl:choose>
					<xsl:when test="contains($varType, ':')">
						<xsl:value-of select="substring-after($varType, ':')"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$varType"/>
					</xsl:otherwise> 
				</xsl:choose>
			</xsl:variable>
			<xsl:if test="$varTypeValue='decimal' or $varTypeValue='int' or $varTypeValue='date'">
				<!-- Add align attribute if need... -->
				<xsl:attribute name="style">
					<xsl:text>text-align: right;</xsl:text>
				</xsl:attribute>
			</xsl:if>
			<!-- Convert value according to type of current element -->
      <xsl:choose>
        <xsl:when test="$varTypeValue='date'">
          <!-- ... convert to russian date format -->
					<xsl:value-of select="concat(substring-after(substring-after($prmValue, '-'), '-'),'.',substring-before(substring-after($prmValue, '-'), '-'), '.', substring-before($prmValue,'-'))" />
				</xsl:when>
        <!-- ... insert your code here for any other types... -->
        <xsl:otherwise>
          <!-- ... otherwise: simple copy of current value -->
          <xsl:value-of disable-output-escaping="yes" select="$prmValue" />
        </xsl:otherwise> 
      </xsl:choose>
    </td>
  </xsl:template>

</xsl:stylesheet>