<?xml version="1.0" encoding="utf-8"?>
<!-- File: genXmlFromDocument.xslt -->
<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xmlns:msxsl="urn:schemas-microsoft-com:xslt"
		xmlns:msdata="urn:schemas-microsoft-com:xml-msdata"
		xmlns:xsd="http://www.w3.org/2001/XMLSchema"
		xmlns:ve="http://schemas.openxmlformats.org/markup-compatibility/2006"
		xmlns:o="urn:schemas-microsoft-com:office:office"
		xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships"
		xmlns:m="http://schemas.openxmlformats.org/officeDocument/2006/math"
		xmlns:v="urn:schemas-microsoft-com:vml"
		xmlns:wp="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing"
		xmlns:w10="urn:schemas-microsoft-com:office:word"
		xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main"
		xmlns:wne="http://schemas.microsoft.com/office/word/2006/wordml"
		exclude-result-prefixes="xsi msxsl msdata xsd ve o r m v wp w10 w wne">

	<xsl:output method="xml" version="1.0"
		encoding="utf-8" omit-xml-declaration="yes" />

	<!-- Current encoding to replace in output xml-PI -->
	<xsl:param name="prmEncoding">
    <xsl:text>utf-8</xsl:text>
  </xsl:param>

	<!-- Schema name -->
	<xsl:param name="prmSchemaUri">
    <xsl:text>Schema_Employee</xsl:text>
  </xsl:param>

	<!-- xsd:schema as first child element in output -->
	<xsl:param name="prmDocXsd" />

	<!-- Get attribute w:element form request w:customXml element in input -->
	<xsl:variable name="varRoot">
		<xsl:value-of select="/./w:document/w:body//w:customXml[@w:uri=$prmSchemaUri]/@w:element"/>
	</xsl:variable>

	<xsl:template match="/">
		<xsl:text disable-output-escaping="yes"><![CDATA[<?xml version="1.0" encoding="]]></xsl:text>
		<xsl:value-of select="$prmEncoding" />
		<xsl:text disable-output-escaping="yes"><![CDATA[" standalone="yes"?>]]>&#xD;&#xA;</xsl:text>
		<xsl:choose>
			<xsl:when test="$varRoot">
				<!-- Generate root element into output -->
				<xsl:element name="{$varRoot}">
          <xsl:if test="$prmDocXsd">
						<!-- Insert schema element into output -->
						<xsl:copy-of select="$prmDocXsd"/>
          </xsl:if>
					<!-- ... continue parsing for childrens of request w:customXml element ... -->
					<xsl:apply-templates select="/./w:document/w:body//w:customXml[@w:uri=$prmSchemaUri]/*" />
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<!-- Error: request element w:customXml not found -->
				<xsl:element name="{concat($prmSchemaUri, '_Not_Found')}" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Parsing any current input... -->
	<xsl:template match="@*|node()">
		<xsl:variable name="varName">
			<xsl:value-of select="name()" />
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="string-length($varName)>0">
				<xsl:choose>
					<xsl:when test="$varName='w:customXml'">
						<!-- Parsing <w:customXml...>...</w:customXml> -->
						<xsl:variable name="varElementName">
							<xsl:value-of select="@w:element" />
						</xsl:variable>
						<xsl:variable name="varChildName">
							<xsl:value-of select="name(child::*[1])"/>
						</xsl:variable>
						<!-- Open <{@w:element}...>... -->
						<xsl:element name="{@w:element}">
							<xsl:choose>
								<xsl:when test="$varElementName=$varRoot or $varChildName='w:tr'">
									<!-- ... for any group: continue parsing for child & as child ... -->
									<xsl:apply-templates select="@*|node()" />
								</xsl:when>
								<xsl:otherwise>
									<!-- ... get xml-value for current w:customXml element -->
									<xsl:call-template name="getXmlValue">
										<xsl:with-param name="prmElementName" select="$varElementName" />
										<xsl:with-param name="prmValue" select="normalize-space(string(child::*))" />
									</xsl:call-template>
								</xsl:otherwise> 
							</xsl:choose>
							<!-- Close ... </{@w:element}> -->
						</xsl:element>
					</xsl:when>
					<xsl:otherwise>
						<xsl:apply-templates select="@*|node()" />
					</xsl:otherwise> 
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="@*|node()" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Generate value of w:customXml element -->
	<xsl:template name="getXmlValue">
		<xsl:param name="prmElementName" />
		<xsl:param name="prmValue" />
		<xsl:choose>
			<xsl:when test="$prmDocXsd">
				<!-- Try get element type from xsd-schema... -->
				<xsl:variable name="varType">
					<xsl:choose>
						<xsl:when test="msxsl:node-set($prmDocXsd)/.//xsd:element[@name=$prmElementName]/@type">
							<xsl:value-of select="msxsl:node-set($prmDocXsd)/.//xsd:element[@name=$prmElementName]/@type"/>
						</xsl:when>
						<xsl:when test="msxsl:node-set($prmDocXsd)/.//xsd:element[@name=$prmElementName]/xsd:simpleType/xsd:restriction/@base">
							<xsl:value-of select="msxsl:node-set($prmDocXsd)/.//xsd:element[@name=$prmElementName]/xsd:simpleType/xsd:restriction/@base"/>
						</xsl:when>
						<!-- ... insert your code here for any other types... -->
						<xsl:otherwise>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<!-- Convert current value according to element type... -->
				<xsl:choose>
					<xsl:when test="$varType='xsd:date'">
						<!-- ... for date value: convert element value to xml-date format into ouptut -->
						<xsl:value-of select="concat(substring-after(substring-after($prmValue, '.'), '.'), '-', substring-before(substring-after($prmValue, '.'), '.'), '-', substring-before($prmValue, '.'))" />
					</xsl:when>
					<!-- ... insert your code here for any other types... -->
					<xsl:otherwise>
						<!-- ... for otherwise case: simple copy current value into ouptut -->
						<xsl:value-of select="$prmValue" />
					</xsl:otherwise> 
				</xsl:choose> 
			</xsl:when>
			<xsl:otherwise>
				<!-- ... for otherwise case: simple copy current value into ouptut -->
				<xsl:value-of select="$prmValue" />
			</xsl:otherwise> 
		</xsl:choose>
	</xsl:template>

</xsl:stylesheet> 
