#define RUS_LANG

using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Win32;
using System.Xml;
using System.Xml.Xsl;
using System.IO;
using System.IO.Packaging;

namespace docx2xml
{
	class docx2xml
	{
		const string PATH_DOCX = @"..\..\..\..\DOCX\";
		const string PATH_XSLT = @"..\..\..\..\XSLT\";
        const string PATH_XML = @"..\..\..\..\XML\";
        const string XSLT_FILE = "genXmlFromDocument.xslt";
        const string XML_OUTPUT_FILE = "documentData.xml";
        const string URI_SETTINGS_XML = "/word/settings.xml";
		const string URI_DOCUMENT_XML = "/word/document.xml";
		const string KEY_SCHEMA_LIBRARY = @"Software\Microsoft\Schema Library";
		const string LOCATION_NAME_VALUE = "Location";
        const string USE_ENCODING = "utf-8";
		const bool USE_TRANSFORM_DEBUG = false;
		const string XML_PI = "<?xml version=\"1.0\" encoding=\"" + USE_ENCODING + "\" standalone=\"yes\"?>\r\n";
#if RUS_LANG
        const string TEMPLATE_DOCX = " VFP-   MS Word 2007.docx";
		const string ERR_LABEL_OUTPUT = "***  : ";
		const string ERR_DIR_NOTFOUND = "  : ";
		const string ERR_FILE_NOTFOUND = "  : ";
        const string ERR_EMPTY_XML = " xml- : ";
        const string ERR_EMPTY_XSLTRES = " xslt-  : ";
		const string ERR_CALL_GENDOCX = " xml2docx.ex    .";
		const string MSG_PRESS_ANY_KEY = "    ...";
		const string MSG_SUCCESS_RESUT = "  !";
#else
        const string TEMPLATE_DOCX = "Example VFP-table in MS Word 2007.docx";
		const string ERR_LABEL_OUTPUT = "*** Error appears: ";
        const string ERR_DIR_NOTFOUND = "Not found directory: ";
        const string ERR_FILE_NOTFOUND = "Not found file: ";
        const string ERR_EMPTY_XML = "Empty xml-data for: ";
        const string ERR_EMPTY_XSLTRES = "Empty result of xslt-transformation for: ";
		const string ERR_CALL_GENDOCX = "Call xml2docx.ex to generating this document.";
		const string MSG_PRESS_ANY_KEY = "Press any key to continue...";
        const string MSG_SUCCESS_RESUT = "Process was successfully completed!";
#endif
		static void Main(string[] args)
		{
			RegistryKey rkDocxAsValue = null; 
			RegistryKey rkDocxAttachedSchema = null;
			RegistryKey rkSchemaLibrary = null;
			RegistryKey rkCurrentUser = null;
			bool bIsResult = false;

			try
			{
				///////////////////////////////////////////////////////////////////////
				// Check existence of external files
				if (!Directory.Exists(PATH_XML))
				{
					throw (new DirectoryNotFoundException(ERR_DIR_NOTFOUND + Path.GetFullPath(PATH_XML)));
				}
				if (!Directory.Exists(PATH_DOCX))
				{
					throw (new DirectoryNotFoundException(ERR_DIR_NOTFOUND + Path.GetFullPath(PATH_DOCX)));
				}
				if (!Directory.Exists(PATH_XSLT))
				{
					throw (new DirectoryNotFoundException(ERR_DIR_NOTFOUND + Path.GetFullPath(PATH_XSLT)));
				}

                ///////////////////////////////////////////////////////////////////////
				// Get docx-file from My Documents folder
				string DocxFolder = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
				string FileDocx = DocxFolder + @"\_" + TEMPLATE_DOCX;
				if (!File.Exists(FileDocx))
				{
					throw (new FileNotFoundException(ERR_FILE_NOTFOUND + FileDocx + ". " + ERR_CALL_GENDOCX));
				}
                string FileXslt = Path.GetFullPath(PATH_XSLT + XSLT_FILE);
                if (!File.Exists(FileXslt))
                {
                    throw (new FileNotFoundException(ERR_FILE_NOTFOUND + FileXslt));
                }

                Encoding enc = Encoding.GetEncoding(USE_ENCODING);
                Uri uriWordSettingsXml = new Uri(URI_SETTINGS_XML, UriKind.Relative);
				Uri uriWordDocumentXml = new Uri(URI_DOCUMENT_XML, UriKind.Relative);
                string strXmlDoc = System.String.Empty;
                using (Package package = Package.Open(FileDocx, FileMode.Open))
				{
					string DocxAttachedSchema = System.String.Empty;

					///////////////////////////////////////////////////////////////////////
                    // Get PackagePart of template for URI_SETTINGS_XML(/word/settings.xml)
                    PackagePart packagePart = package.GetPart(uriWordSettingsXml);
                    // ... and its stream in Read mode
					using (Stream ppStrmSettingsXml = packagePart.GetStream(FileMode.Open, FileAccess.Read))
					{
						///////////////////////////////////////////////////////////////////////
						// Load stream as xml-document
						XmlDocument xmlDocSettings = new XmlDocument();
						xmlDocSettings.Load(ppStrmSettingsXml);
						XmlElement xeRoot = xmlDocSettings.DocumentElement;
						string nsp_w = xeRoot.GetNamespaceOfPrefix("w"); 
						XmlNamespaceManager nsMngr = new XmlNamespaceManager(xmlDocSettings.NameTable);
						if (nsp_w != null && nsp_w.Length > 0)
						{
							nsMngr.AddNamespace("w", nsp_w); 
						}

						///////////////////////////////////////////////////////////////////////
						// Get AttachedSchema name
						XmlNode xnAttachedSchema = xeRoot.SelectSingleNode("w:attachedSchema", nsMngr);
						if (xnAttachedSchema != null && xnAttachedSchema.Attributes.Count > 0)
						{
							XmlAttribute xaVal = xnAttachedSchema.Attributes["w:val"];
							if (xaVal != null)
							{
								DocxAttachedSchema = xaVal.Value; 
							}
						}
						ppStrmSettingsXml.Close(); 
					}
					string PathSchemaLocation = System.String.Empty;
					if (DocxAttachedSchema.Length > 0)
					{
						///////////////////////////////////////////////////////////////////////////////
						// Get Location for XSD-schema from Microsoft Schema Library in system rgistry
						rkCurrentUser = Registry.CurrentUser;
						// Get RegistryKey: Microsoft Schema Library
						rkSchemaLibrary = rkCurrentUser.OpenSubKey(KEY_SCHEMA_LIBRARY, false);
						// Get RegistryKey: AttachedSchema by name
						rkDocxAttachedSchema = rkSchemaLibrary.OpenSubKey(DocxAttachedSchema, false);
						// Get RegistryKey: first child of AttachedSchema
						string[] asSubKeys = rkDocxAttachedSchema.GetSubKeyNames();
						rkDocxAsValue = rkDocxAttachedSchema.OpenSubKey(asSubKeys[0], false);
						// Get Location
						PathSchemaLocation = (string)rkDocxAsValue.GetValue(LOCATION_NAME_VALUE);

						/////////////////////////////////////
						// Close all RegistryKeys
						rkDocxAsValue.Close();
						rkDocxAsValue = null;
						rkDocxAttachedSchema.Close();
						rkDocxAttachedSchema = null;
						rkSchemaLibrary.Close();
						rkSchemaLibrary = null;
						rkCurrentUser.Close();
						rkCurrentUser = null;
					}

					///////////////////////////////////////////////////////////////////////
                    // Get PackagePart of template for URI_DOCUMENT_XML(/word/document.xml)
                    packagePart = package.GetPart(uriWordDocumentXml);
                    // ... and its stream in Read mode
					using (Stream ppStrmDocumentXml = packagePart.GetStream(FileMode.Open, FileAccess.Read))
					{
						///////////////////////////////////////////////////////////////////////
						// Load stream as xml-document
						XmlDocument xmlDocumentXml = new XmlDocument();
						xmlDocumentXml.Load(ppStrmDocumentXml);
						
						///////////////////////////////////////////////////////////////////////////
						// Load XSD-schema from file SchemaLocation (as info from system registry)
						XmlDocument xmlDocXsd = null;
                        if (File.Exists(PathSchemaLocation))
                        {
                            xmlDocXsd = new XmlDocument();
                            xmlDocXsd.Load(PathSchemaLocation); 
                        }

						///////////////////////////////////////////////////////////////////////////////
						// Prepare and execute the xslt-transformation 
						// with document.xml and by using xslt-file FileXslt
						XslCompiledTransform xsltXmlTransform = new XslCompiledTransform(USE_TRANSFORM_DEBUG);
                        XsltArgumentList xsltArgList = new XsltArgumentList();
                        xsltArgList.AddParam("prmEncoding", "", USE_ENCODING);
                        if (DocxAttachedSchema.Length > 0)
                        {
                            xsltArgList.AddParam("prmSchemaUri", "", DocxAttachedSchema);
                        }
                        if (xmlDocXsd != null && xmlDocXsd.DocumentElement != null)
                        {
                            xsltArgList.AddParam("prmDocXsd", "", xmlDocXsd.DocumentElement.CreateNavigator());
                        }
                        xsltXmlTransform.Load(FileXslt);
                        using (MemoryStream msXmlDoc = new MemoryStream())
                        {
                            using (XmlTextWriter xtwXmlDoc = new XmlTextWriter(msXmlDoc, enc))
                            {
                                xsltXmlTransform.Transform(xmlDocumentXml.CreateNavigator(), xsltArgList, xtwXmlDoc);
                                strXmlDoc = enc.GetString(msXmlDoc.GetBuffer(), 0, (int)msXmlDoc.Length);
                                xtwXmlDoc.Close();
                            }
                            msXmlDoc.Close();
                        }
                        int nPos = strXmlDoc.IndexOf('<');
                        if (nPos == (-1))
                        {
                            throw (new NullReferenceException(ERR_EMPTY_XSLTRES + FileXslt));
                        }
                        strXmlDoc = strXmlDoc.Substring(nPos);
                        nPos = strXmlDoc.IndexOf("<?xml");
                        if (nPos == (-1))
                        {
                            strXmlDoc = XML_PI + strXmlDoc;
                        }
						ppStrmDocumentXml.Close(); 
					}
					package.Close(); 
				}

                //////////////////////////////////////
                // Save result
                string xmlOutputFile = Path.GetFullPath(PATH_XML + XML_OUTPUT_FILE);
                if (strXmlDoc.Length > 0)
                {
                    if (File.Exists(xmlOutputFile))
                    {
                        File.Delete(xmlOutputFile); 
                    }
                    XmlDocument xmlDocOut = new XmlDocument();
                    xmlDocOut.LoadXml(strXmlDoc);
                    xmlDocOut.Save(xmlOutputFile);
                    bIsResult = File.Exists(xmlOutputFile);
                }
                else
                {
                    throw (new FileNotFoundException(ERR_EMPTY_XML + xmlOutputFile));
                }
			}
			catch (Exception ex)
			{
				Console.WriteLine(ERR_LABEL_OUTPUT + ex.ToString());
			}

			/////////////////////////////////////
			// Close all RegistryKeys
			if (rkDocxAsValue != null)
			{
				rkDocxAsValue.Close(); 
				rkDocxAsValue = null; 
			}
			if (rkDocxAttachedSchema != null)
			{
				rkDocxAttachedSchema.Close(); 
				rkDocxAttachedSchema = null;
			}
			if (rkSchemaLibrary != null)
			{
				rkSchemaLibrary.Close(); 
				rkSchemaLibrary = null;
			}
			if (rkCurrentUser != null)
			{
				rkCurrentUser.Close(); 
				rkCurrentUser = null;
			}
			if (bIsResult)
			{
				Console.WriteLine(MSG_SUCCESS_RESUT);
			}
			Console.WriteLine(MSG_PRESS_ANY_KEY);
			Console.ReadKey();
		}
	}
}
