﻿#define RUS_LANG
#define TMP_OUT

using System;
using System.Collections.Generic;
using System.Text;

using System.Xml;
using System.Xml.Xsl;
using System.IO;
using System.IO.Packaging;
using System.Diagnostics;  

namespace xml2docx
{
    class xml2docx
    {
        const string PATH_XML = @"..\..\..\..\XML\";
        const string PATH_DOCX = @"..\..\..\..\DOCX\";
        const string PATH_XSLT = @"..\..\..\..\XSLT\";
        const string XML_DATA_FILE = "employee.XML";
        const string XSLT_FILE = "genXsltFromTemplate.xslt";
        const string XSLT_FILE_FIRST_RESULT = "getXmlDocument.xslt";
        const string XML_FILE_SECOND_RESULT = "document.xml";
        const string URI_DOCUMENT_XML = "/word/document.xml";
        const string USE_ENCODING = "utf-8";
        const string XML_PI = "<?xml version=\"1.0\" encoding=\"" + USE_ENCODING + "\" standalone=\"yes\"?>\r\n";
#if RUS_LANG
        const string TEMPLATE_DOCX = "Пример VFP-таблицы в виде MS Word 2007.docx";
		const string ERR_LABEL_OUTPUT = "*** Возникла ошибка: ";
		const string ERR_DIR_NOTFOUND = "Не найден каталог: ";
        const string ERR_FILE_NOTFOUND = "Не найден файл: ";
        const string ERR_EMPTY_XML = "Нет xml-данных для: ";
        const string ERR_EMPTY_XSLTRES = "Результат xslt-преобразований пуст для: ";
        const string DOC_PROP_DESCRIPTION = "Документ создан на основе шаблона и программно заполнен данными из xml-файла.";
        const string DOC_PROP_SUBJECT = "Использование Open XML формата в MS Word 2007";
        const string MSG_SUCCESS_RESUT = "Процесс успешно завершён! Ожидание открытия полученного нового документа в MS Word 2007...";
		const string MSG_PRESS_ANY_KEY = "Нажмите любую клавишу для продолжения...";
#else
        const string TEMPLATE_DOCX = "Example VFP-table in MS Word 2007.docx";
		const string ERR_LABEL_OUTPUT = "*** Error appears: ";
        const string ERR_DIR_NOTFOUND = "Not found directory: ";
        const string ERR_FILE_NOTFOUND = "Not found file: ";
        const string ERR_EMPTY_XML = "Empty xml-data for: ";
        const string ERR_EMPTY_XSLTRES = "Empty result of xslt-transformation for: ";
        const string DOC_PROP_DESCRIPTION = "The document is created on the basis of template and is filled by data from a xml-file from code program.";
        const string DOC_PROP_SUBJECT = "Use Open XML format in MS Word 2007";
        const string MSG_SUCCESS_RESUT = "Process was successfully completed! Waiting of opening of the received new document in MS Word 2007...";
		const string MSG_PRESS_ANY_KEY = "Press any key to continue...";
#endif
		static void Main(string[] args)
        {
            bool bStartAppWord = false;
            try
            {
                ///////////////////////////////////////////////////////////////////////
                // Check existence of external files
                if (!Directory.Exists(PATH_XML))
                {
                    throw (new DirectoryNotFoundException(ERR_DIR_NOTFOUND + Path.GetFullPath(PATH_XML)));
                }
                if (!Directory.Exists(PATH_DOCX))
                {
                    throw (new DirectoryNotFoundException(ERR_DIR_NOTFOUND + Path.GetFullPath(PATH_DOCX)));
                }
                if (!Directory.Exists(PATH_XSLT))
                {
                    throw (new DirectoryNotFoundException(ERR_DIR_NOTFOUND + Path.GetFullPath(PATH_XSLT)));
                }
                string FileXmlData = Path.GetFullPath(PATH_XML + XML_DATA_FILE);
                if (!File.Exists(FileXmlData))
                {
                    throw (new FileNotFoundException(ERR_FILE_NOTFOUND + FileXmlData));
                }
                string FileTemplateDocx = Path.GetFullPath(PATH_DOCX + TEMPLATE_DOCX);
                if (!File.Exists(FileTemplateDocx))
                {
                    throw (new FileNotFoundException(ERR_FILE_NOTFOUND + FileTemplateDocx));
                }
                string FileGenXsltFromTemplate = Path.GetFullPath(PATH_XSLT + XSLT_FILE);
                if (!File.Exists(FileTemplateDocx))
                {
                    throw (new FileNotFoundException(ERR_FILE_NOTFOUND + FileGenXsltFromTemplate));
                }

                ///////////////////////////////////////////////////////////////////////
                // Copy template to My Documents folder
                string OutputFolder = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
                string OutputFileDocx = OutputFolder + @"\_" + TEMPLATE_DOCX;
                
                File.Copy(FileTemplateDocx, OutputFileDocx, true);
                if (!File.Exists(OutputFileDocx))
                {
                    throw (new FileNotFoundException(ERR_FILE_NOTFOUND + OutputFileDocx));
                }

                ///////////////////////////////////////////////////////////////////////
                // Load xml-data file and get name of its root element
                XmlDocument xmlDocData = new XmlDocument();
                xmlDocData.Load(FileXmlData);
                if (xmlDocData.DocumentElement == null || xmlDocData.DocumentElement.ChildNodes.Count == 0)
                {
                    throw (new NullReferenceException(ERR_EMPTY_XML + FileXmlData));
                }
                string rootElementName = xmlDocData.DocumentElement.Name;

                ///////////////////////////////////////////////////////////////////////
                // Get package copy of template and prepare for its processing
                Encoding enc = Encoding.GetEncoding(USE_ENCODING);
                string strXmlDoc = System.String.Empty;
                string docxTitle = System.String.Empty;
                Uri uriWordDocXml = new Uri(URI_DOCUMENT_XML, UriKind.Relative);
                using (Package package = Package.Open(OutputFileDocx, FileMode.Open))
                {
                    ///////////////////////////////////////////////////////////////////////
                    // Get PackagePart of template for URI_DOCUMENT_XML(/word/document.xml)
                    PackagePart packagePart = package.GetPart(uriWordDocXml);
                    // ... and its stream in ReadWrite mode
                    using (Stream ppStrmXmlDoc = packagePart.GetStream(FileMode.Open, FileAccess.ReadWrite))
                    {
                        ///////////////////////////////////////////////////////////////////////////////
                        // Prepare and execute the first xslt-transformation 
                        // with document.xml and by using xslt-file FileGenXsltFromTemplate
                        XmlDocument xmlDoc = new XmlDocument();
                        xmlDoc.Load(ppStrmXmlDoc);
                        XsltArgumentList xsltArgumentList = new XsltArgumentList();
                        xsltArgumentList.AddParam("prmRootElement", "", rootElementName);
                        xsltArgumentList.AddParam("prmEncoding", "", USE_ENCODING);
                        XslCompiledTransform xslXsltDocTrans = new XslCompiledTransform();
                        xslXsltDocTrans.Load(FileGenXsltFromTemplate);
                        string strXsltDoc = System.String.Empty;
                        using (MemoryStream msXsltDoc = new MemoryStream())
                        {
                            using (XmlTextWriter xtwXsltDoc = new XmlTextWriter(msXsltDoc, enc))
                            {
                                xslXsltDocTrans.Transform(xmlDoc.CreateNavigator(), xsltArgumentList, xtwXsltDoc);
                                strXsltDoc = enc.GetString(msXsltDoc.GetBuffer(), 0, (int)msXsltDoc.Length);
                                xtwXsltDoc.Close();
                            }
                            msXsltDoc.Close();
                        }
                        int nPos = strXsltDoc.IndexOf('<');
                        if (nPos == (-1))
                        {
                            throw (new NullReferenceException(ERR_EMPTY_XSLTRES + FileGenXsltFromTemplate));
                        }
                        strXsltDoc = strXsltDoc.Substring(nPos);
                        nPos = strXsltDoc.IndexOf("<?xml");
                        if (nPos == (-1))
                        {
                            strXsltDoc = XML_PI + strXsltDoc;
                        }

                        ///////////////////////////////////////////////////////////////////////////////
                        // Load result of first xslt-transformation
                        XmlDocument xmlDocXslt = new XmlDocument();
                        xmlDocXslt.LoadXml(strXsltDoc);
#if TMP_OUT
                        xmlDocXslt.Save(Path.GetFullPath(XSLT_FILE_FIRST_RESULT));
#endif
                        ///////////////////////////////////////////////////////////////////////////////
                        // Prepare and execute the second xslt-transformation 
                        // with input xml-data-file and by using first result of xslt-transformation
                        XslCompiledTransform xslXmlDocTrans = new XslCompiledTransform();
                        xslXmlDocTrans.Load(xmlDocXslt.CreateNavigator());
                        using (MemoryStream msXmlDoc = new MemoryStream())
                        {
                            using (XmlTextWriter xtwXmlDoc = new XmlTextWriter(msXmlDoc, enc))
                            {
                                xslXmlDocTrans.Transform(xmlDocData.CreateNavigator(), xtwXmlDoc);
                                strXmlDoc = enc.GetString(msXmlDoc.GetBuffer(), 0, (int)msXmlDoc.Length);
                                xtwXmlDoc.Close();
                            }
                            msXmlDoc.Close();
                        }
                        nPos = strXmlDoc.IndexOf('<');
                        if (nPos == (-1))
                        {
                            throw (new NullReferenceException(ERR_EMPTY_XSLTRES + FileXmlData));
                        }
                        strXmlDoc = strXmlDoc.Substring(nPos);
                        nPos = strXmlDoc.IndexOf("<?xml");
                        if (nPos == (-1))
                        {
                            strXmlDoc = XML_PI + strXmlDoc;
                        }

                        ///////////////////////////////////////////////////////////////////////////////
                        // Load result of second xslt-transformation as xml-document
                        XmlDocument xmlDocOut = new XmlDocument();
                        xmlDocOut.LoadXml(strXmlDoc);
#if TMP_OUT
                        xmlDocOut.Save(Path.GetFullPath(XML_FILE_SECOND_RESULT));
#endif
                        ///////////////////////////////////////////////////////////////////////////////
                        // Replace PackagePart-stream with result of second xslt-transformation
                        //ppStrmXmlDoc.Seek(0, SeekOrigin.Begin);
						ppStrmXmlDoc.Position = 0;
						xmlDocOut.Save(ppStrmXmlDoc);
                        ppStrmXmlDoc.Flush();
                        ppStrmXmlDoc.Close();

                        ///////////////////////////////////////////////////////////////////////////////
                        // Get the text of the first paragraph of the document as title of the document
                        string ns_w = xmlDocOut.DocumentElement.GetNamespaceOfPrefix("w");
                        XmlNamespaceManager nsManager = new XmlNamespaceManager(xmlDocOut.NameTable);
                        nsManager.AddNamespace("w", ns_w);
                        XmlNode xnTitle = xmlDocOut.DocumentElement.SelectSingleNode("//w:customXml[@w:element='" + rootElementName + "']/w:p", nsManager);
                        if (xnTitle != null)
                        {
                            docxTitle = xnTitle.InnerText;
                        }
                    }
                    if (docxTitle.Length > 0)
                    {
                        ///////////////////////////////////////////////////////////////////////////////
                        // Replace title and some other PackageProperties
                        package.PackageProperties.Title = docxTitle;
                        package.PackageProperties.Subject = DOC_PROP_SUBJECT;
                        package.PackageProperties.Description = DOC_PROP_DESCRIPTION;
                    }
                    package.Flush();
                    package.Close();
                }

                ///////////////////////////////////////////////////////////////////////////////
                // Open the new docx file in Word 2007
				Console.WriteLine(MSG_SUCCESS_RESUT);
				Process proc = new Process();
                proc.EnableRaisingEvents = false;
                proc.StartInfo.FileName = "winword";
                proc.StartInfo.Arguments = "\"" + OutputFileDocx + "\"";
                proc.Start();
                bStartAppWord = true;
			}
            catch (Exception ex)
            {
				Console.WriteLine(ERR_LABEL_OUTPUT + ex.ToString());  
            }
            if (!bStartAppWord)
            {
				Console.WriteLine(MSG_PRESS_ANY_KEY);
                Console.ReadKey();
            }
        }
    }
}
